/*

  Render Silhouette edges for triangle soup
  No pre-processing
  No connectivity information required
  
	Code is based on Interactive 3D Graphics paper, April 1999
	"Image Precision Silhouette Edges"
	Ramesh Raskar and Michael Cohen
	
	Ramesh Raskar
	  raskar@cs.unc.edu
	  http://www.cs.unc.edu/~raskar
	  Feb 2000
	  
		
	Mouse buttons :
		  Left : rotate object
		  Middle : translate
		  Right : menu
	Keys
		  Use l,L, z, Z, s, S to change width of silhouettes
		  
	The code is written primarily so that you can take parts of it
			and use in your code. For this purpose, the data-structures,
			file formats and user interface are kept very simple.
	The three functions you should look at are drawsilhouette### ().
			
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <time.h>
#include <math.h>
#include "string.h"

void drawsilhouetteBackFattened();

#include "matrixFunc.h"
#include "otherStuff.h"


GLfloat nAngle = 90.0, nAngle2 = -90.0;
GLfloat xdist = 0, ydist = 0, zdist = 0.0;
int ox, oy;
int buttonState;


int CURRENT_MODEL = CUBE_MODEL;
GLfloat z_near = 1.12;
GLfloat lineWidth = 0.7;
GLfloat scaleT = 0.05;
int charcoalFlag = 0;
int colorObjFlag = 1;
GLfloat cosAngleThreshold = 0.0;
void drawBackFattened();
void drawObject();

int rendermode = SILRIDGE; //initial mode

#define Res 640
int winWidth = Res;
int winHeight = Res*3/4;

//GLfloat eye[3] = {100.0,100.0,-50.0};
GLfloat eye[3] = {0.0,0.0,0.0};
GLfloat light[3] = {0.7, 0.7, 0};

#include "creaseStuff.h"


////////////////////////////////////////////////////////////////////////
/* 
Method 1 :
WireFrame method
Draw front polygons filled
Draw back polygons black wireframe 
*/
void
drawsilhouetteLine()
{
	// Front
    if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
    glCullFace(GL_BACK);
    glEnable(GL_CULL_FACE);
    glDepthMask(GL_TRUE);
    glColor3f(0.7f, 0.7f, 1.0f);
    drawObject();
    
	// Back
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    glPolygonMode(GL_BACK, GL_LINE);
    glDepthFunc(GL_LEQUAL);
    glCullFace(GL_FRONT);
    glDepthMask(GL_FALSE);	/* depth buffer is read only */
    glColor3f(0.1f, 0.1f, 0.5f);
    glLineWidth(lineWidth);
	glPushMatrix();
	if (charcoalFlag == 0) drawObject(); else drawBackFattened();
	glPopMatrix();
	
	cleanGLstate();
}

/* 
Method 2 :
Translation of back facing polygons
Draw front polygons filled
Draw back polygons in black translated fwd towards the camera
*/

void
drawsilhouetteZscaled()
{
	// Front
    if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
    glCullFace(GL_BACK);
    glEnable(GL_CULL_FACE);
    glDepthMask(GL_TRUE);
    glColor3f(0.7f, 0.9f, 1.0f);
	drawObject();
	
	// Back
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    glPolygonMode(GL_BACK, GL_FILL);
    glDepthFunc(GL_LEQUAL);
    glCullFace(GL_FRONT);
    glDepthMask(GL_FALSE);	/* depth buffer is read only */
    glColor3f(0.2f, 0.2f, 0.5f);
	
	glPushMatrix();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( FOV,(float) winWidth/winHeight,z_near,ZFAR); /* create different z grading */
	glMatrixMode(GL_MODELVIEW);
	if (charcoalFlag == 0) drawObject(); else drawBackFattened();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(FOV, (float) winWidth/winHeight,1.0,ZFAR); /* create different z grading */
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
	cleanGLstate();
}

/* 
Method 3 :
View dependent fattening of back facing polygons
Draw front polygons filled
Draw back polygons in black fattened
*/

void
drawsilhouetteBackFattened()
{
	// Front
	if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);  
	//glColor4f(0.8f, 0.8f, 1.f, 1.0f); 
	glColor4f(1.0f, 1.0f, 0.f, 1.0f); 
	drawObject();
	
	// Back
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glPolygonMode(GL_BACK, GL_FILL);
	glDepthFunc(GL_LEQUAL);
	glDisable(GL_CULL_FACE);
	if (colorObjFlag == 0) glColor3f(0.5f, 0.5f, 1.0f); else glColor4f(0.f, 0.f, 0.f, 1.0f);
	
	glDisable(GL_BLEND);
	drawBackFattened();
	
	cleanGLstate();
}


void drawBackFattenedOld()
{
	int i,num;
	GLfloat temp[3];
	GLfloat temp1[3];
	GLfloat eyeVec[3];
	GLfloat eyeT[3], lightT[3];
	GLfloat dotval, dotValLight,eyeDist, orntFactor, cosAngle, cosAlpha,k,k2,kShift = 0.1,k2Shift= 0.0;
	GLfloat modelview_mat[16], iview[16], id_mat[16];
	GLfloat vert01[3], vert12[3], vert20[3];
	GLfloat vert10[3], vert21[3], vert02[3];
	Vec3 E10, E21, E02, N10, N21, N02;
	
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview_mat);
	mat4Invert(modelview_mat, iview);
	mat4Mult(modelview_mat, iview, id_mat);
	
	mat4vec3Mult(iview, eye, eyeT);
	mat4vec3Mult(iview, light, lightT,0);
	num = CURRENT_MODEL;
	
	glBegin(GL_TRIANGLES);
	for (i = 0; i < trisStruct[num].numTris; i ++) {
		dotval = vecDotProduct(lightT, trisStruct[num].tris[i].norm);	
		
		if (charcoalFlag == 0) {
			glColor3f(0.0,0.0,0.0);
			cosAngleThreshold = 0.0;
		}   
		else {
			dotValLight = (1.0 + dotval)/3.0;
			glColor3f(dotValLight, dotValLight, dotValLight);
			cosAngleThreshold = 0.2;
		}   
		
		vecSub( eyeT,trisStruct[num].tris[i].midpt, eyeVec);
		eyeDist = vecMagn(eyeVec);
		vecNormalize(eyeVec);
		cosAngle = vecDotProduct(eyeVec,trisStruct[num].tris[i].norm);
		if (cosAngle <  cosAngleThreshold) /* back face culling */{
			if (cosAngle < 0) cosAngle = - cosAngle;
			orntFactor = (eyeDist/10.0)*(sqrt(1.0 - cosAngle*cosAngle) + kShift);
			k = orntFactor*scaleT;
			
			/* find three edges and normal to those edges in triangles plane*/
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d, E10);
			vecNormalize(E10);
			vecCross(E10, trisStruct[num].tris[i].norm, N10);
			vecNormalize(N10);
			
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, E21);
			vecNormalize(E21);
			vecCross(E21, trisStruct[num].tris[i].norm, N21);
			vecNormalize(N21);
			
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d, E02);
			vecNormalize(E02);
			vecCross(E02,trisStruct[num].tris[i].norm, N02);
			vecNormalize(N02);
			
			/* push all three edges out along the edge-normals */
			cosAlpha = vecDotProduct(eyeVec,E10);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha) + k2Shift;
			vecScale(N10, k*k2,temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,temp1, vert01);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,temp1, vert10);
			glVertex3fv(vert01);
			glVertex3fv(vert10);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			cosAlpha = vecDotProduct(eyeVec,E21);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha)  + k2Shift;
			vecScale(N21, k*k2, temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,temp1, vert12);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,temp1, vert21);
			glVertex3fv(vert12);
			glVertex3fv(vert21);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			cosAlpha = vecDotProduct(eyeVec,E02);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha) + k2Shift;
			vecScale(N02, k*k2, temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,temp1, vert20);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,temp1, vert02);
			glVertex3fv(vert20);
			glVertex3fv(vert02);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			
			/* connect the fan */
            glVertex3fv(vert10);
			glVertex3fv(vert12);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			glVertex3fv(vert21);
			glVertex3fv(vert20);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			glVertex3fv(vert02);
			glVertex3fv(vert01);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
		}
	}
    glEnd();
}

void drawBackFattened()
{
	int i,num;
	GLfloat temp[3];
	GLfloat temp1[3];
	GLfloat eyeVec[3];
	GLfloat eyeT[3], lightT[3];
	GLfloat dotval, dotValLight,eyeDist, orntFactor, cosAngle, cosAlpha,k,k2,kShift = 0.3,k2Shift= 0.0;
	GLfloat modelview_mat[16], iview[16], id_mat[16];
	GLfloat vert01[3], vert12[3], vert20[3];
	GLfloat vert10[3], vert21[3], vert02[3];
	Vec3 E10, E21, E02, N10, N21, N02;
	
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview_mat);
	mat4Invert(modelview_mat, iview);
	mat4Mult(modelview_mat, iview, id_mat);
	
	mat4vec3Mult(iview, eye, eyeT);
	mat4vec3Mult(iview, light, lightT,0);
	num = CURRENT_MODEL;
	
	glBegin(GL_TRIANGLES);
	for (i = 0; i < trisStruct[num].numTris; i ++) {
		dotval = vecDotProduct(lightT, trisStruct[num].tris[i].norm);	
		
		if (charcoalFlag == 0) {
			glColor3f(0.0,0.0,0.0);
			cosAngleThreshold = 0.0;
		}   
		else {
			dotValLight = (1.0 + dotval)/3.0;
			glColor3f(dotValLight, dotValLight, dotValLight);
			cosAngleThreshold = 0.2;
		}   
		
		vecSub( eyeT,trisStruct[num].tris[i].midpt, eyeVec);
		eyeDist = vecMagn(eyeVec);
		vecNormalize(eyeVec);
		cosAngle = vecDotProduct(eyeVec,trisStruct[num].tris[i].norm);
		if (cosAngle <  cosAngleThreshold) /* back face culling */{
			if (cosAngle < 0) cosAngle = - cosAngle;
			orntFactor = (eyeDist/10.0)/(cosAngle + kShift);
			k = orntFactor*scaleT;
			
			/* find three edges and normal to those edges in triangles plane*/
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d, E10);
			vecNormalize(E10);
			vecCross(E10, trisStruct[num].tris[i].norm, N10);
			vecNormalize(N10);
			
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, E21);
			vecNormalize(E21);
			vecCross(E21, trisStruct[num].tris[i].norm, N21);
			vecNormalize(N21);
			
			vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d, E02);
			vecNormalize(E02);
			vecCross(E02,trisStruct[num].tris[i].norm, N02);
			vecNormalize(N02);
			
			/* push all three edges out along the edge-normals */
			cosAlpha = vecDotProduct(eyeVec,E10);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha) + k2Shift;
			vecScale(N10, k*k2,temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,temp1, vert01);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,temp1, vert10);
			glVertex3fv(vert01);
			glVertex3fv(vert10);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			cosAlpha = vecDotProduct(eyeVec,E21);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha)  + k2Shift;
			vecScale(N21, k*k2, temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d,temp1, vert12);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,temp1, vert21);
			glVertex3fv(vert12);
			glVertex3fv(vert21);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			cosAlpha = vecDotProduct(eyeVec,E02);
			if (cosAlpha < 0) cosAlpha = - cosAlpha;
			k2 = sqrt(1.0 - cosAlpha*cosAlpha) + k2Shift;
			vecScale(N02, k*k2, temp1);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d,temp1, vert20);
			vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d,temp1, vert02);
			glVertex3fv(vert20);
			glVertex3fv(vert02);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			
			/* connect the fan */
            glVertex3fv(vert10);
			glVertex3fv(vert12);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			glVertex3fv(vert21);
			glVertex3fv(vert20);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
			glVertex3fv(vert02);
			glVertex3fv(vert01);
			glVertex3fv(trisStruct[num].tris[i].midpt);
			
		}
	}
    glEnd();
}

void perturb ()
{
	int i;
	int	num = CURRENT_MODEL;
	double x, y, z, shiftx, shifty, shiftz;
	//double fac = 1.0001*rand()/RAND_MAX;
	double fac = 0.001 + 0.0001*rand()/RAND_MAX;
	GLfloat normal[3];
	GLfloat temp1[3], temp2[3];


	for (i= 0 ; i < vertStruct[num].numVerts; i++) {
			x = vertStruct[num].verts[i].d[0]/300.0 - 0.5;
			y = vertStruct[num].verts[i].d[1]/300.0 - 0.5;
			z = vertStruct[num].verts[i].d[2]/300.0 - 0.5;
			shiftx = fac*( sin(10*y)*cos(11*x)*sin(z));
			shifty = fac*(cos(11*z)*sin(19*x));
			shiftz = fac*(sin(2*x));
			/*shiftx = fac*( -x*x + y*y + z*z);
			shifty = fac*(2*x + y*y - z);
			shiftz = fac*(x - y + z*z);*/
			vertStruct[num].verts[i].d[0] += shiftx*300.0;
			vertStruct[num].verts[i].d[1] += shifty*300.0;
			vertStruct[num].verts[i].d[2] += shiftz*300.0;
	}

	for (i= 0 ; i < trisStruct[num].numTris; i++) {
		//printf("i %d\n", i);
		// Assign normal and midpt
		vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d , 
			vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d, temp1) ;
		vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d ,
			vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, temp2);
		vecCross(temp1, temp2, normal);
		vecNormalize(normal);
		vecAssign(normal, trisStruct[num].tris[i].norm);
		vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d ,
			vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, temp1);
		vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d ,temp1, temp2);
		vecScale(temp2, 1.0/3.0,trisStruct[num].tris[i].midpt);
		
	}
}
	

void drawObject()
{
	int i,num;
	num = CURRENT_MODEL;
	glDisable(GL_BLEND);
	glBegin(GL_TRIANGLES);
	for (i = 0; i < trisStruct[num].numTris; i ++) {
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d);
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d);
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d);
	}
	glEnd();
	glEnable(GL_BLEND);
}

void menu(int mode)
{
	switch(mode) {
	case SIL_FAT:
	case OBJ_BACK:
	case OBJ_FRONT:
	case SIL_LINE:
	case SIL_ZSCALED:
	case OBJ_WIRED:
	case OBJ_WIREFILLED:
	case SHARP:
	case VALLEY:
	case SILRIDGE:
	case SILRIDGEVALLEY:
	case INTERSECT:
	case VALLEYINTERSECT:
		rendermode = mode;
		break;
	case SIL_AND_OBJ:
		colorObjFlag = 1 - colorObjFlag;
		break;
	case CHARCOAL:
		charcoalFlag = 1 - charcoalFlag;
		rendermode = SIL_FAT;
		printf("Charcoal Mode %d\n", charcoalFlag );
		break;
	case FACE_MODE :
		CURRENT_MODEL = FACE_MODEL;
		break;
	case VENUS_MODE :
		CURRENT_MODEL = VENUS_MODEL;
		break;
	case CUBE_MODE :
		CURRENT_MODEL = CUBE_MODEL;
		charcoalFlag = 0;
		break;
	case BUNNY_MODE :
		CURRENT_MODEL = BUNNY_MODEL;
		charcoalFlag = 0;
		break;
	}
	glutPostRedisplay();
}




void
reshape(int wid, int ht)
{
	glViewport(0, 0, wid, ht);
	winWidth = wid;
	winHeight = ht;
	glutPostRedisplay();
}

int diffTime = 0, countTime = 0;

void
display()
{
	int tstart, tstop;
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

	tstart = timeGetTime();
	
	glPushMatrix();

	// very simple user interface
	glTranslatef (xdist, ydist, zdist);	
    glRotatef ((GLfloat) nAngle, 0.0f, 1.0f, 0.0f);
    glRotatef ((GLfloat) nAngle2, 1.0f, 0.0f, 0.0f);
		
	switch(rendermode) {
	case SIL_LINE:
		drawsilhouetteLine();
		break;
	case SIL_ZSCALED:
		drawsilhouetteZscaled();
		break;
	case SIL_FAT:
		drawsilhouetteBackFattened();
		if (sharpFlag) drawSharps();
		break;
	case SHARP : 
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawSharps();
		break;
	case SILRIDGE :
		//sharpFlag = 1;
		drawsilhouetteBackFattened();
		drawSharps();
		//sharpFlag = 0;
		break;
	case SILRIDGEVALLEY :
		sharpFlag = 1;
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValley2();
		sharpFlag = 0;
		break;
	case VALLEY :
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValley2();
		break;
	case INTERSECT :
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawIntersect();
		break;
	case VALLEYINTERSECT : 
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValleyIntersect();
		break;
	case OBJ_BACK:
		glCullFace(GL_FRONT);
		glEnable(GL_CULL_FACE);
		glColor3f(0.8,0.8,1.0);
		charcoalFlag = 1; 
		drawBackFattened();
		charcoalFlag = 0; 
		break;
	case OBJ_FRONT:
		glColor3f(0.8,0.8,1.0);
		glCullFace(GL_BACK);
		glEnable(GL_CULL_FACE);
		drawObject();
		break;
	case OBJ_WIRED:
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glColor3f(0.0,0.0,0.0);
		glLineWidth(1.0);
		drawObject();
		cleanGLstate();
		break;
	case OBJ_WIREFILLED:
		glEnable(GL_CULL_FACE);
		glColor3f(0.99,0.99,1.0);
		glCullFace(GL_FRONT);
		glPolygonMode(GL_BACK, GL_FILL);
		drawObject();
		glColor3f(0.0,0.0,0.0);
		glCullFace(GL_BACK);
		glLineWidth(1.0);
		glPolygonMode(GL_FRONT, GL_LINE);
		drawObject();
		cleanGLstate();
		break;
	}
    glPopMatrix();
	glFinish();
	tstop = timeGetTime();
	
	countTime ++; diffTime += tstop - tstart;
    glutSwapBuffers();
	//printf("time = %d\n", tstop- tstart);
	if (countTime == 30) {  
		//printf("fps = %f\n", (float) 30*1000/diffTime); 
		countTime = 0; diffTime = 0;} 
}



void key(unsigned char key, int x, int y)
{
	switch(key) {
	case 'c' :
		charcoalFlag = 1 - charcoalFlag; 
		printf("Charcoal Mode %d\n", charcoalFlag);
		break;
	case 'z' :
		z_near += 0.01;	printf("z_near %f\n", z_near);
		break;
	case 'Z' :
		z_near -= 0.01;	printf("z_near %f\n", z_near);
		break;
	case 's' :
		scaleT += 0.01;	printf("scaleT %f\n", scaleT);
		break;
	case 'S' :
		scaleT -= 0.01;	printf("scaleT %f\n", scaleT);
		break;
	case 'l' :
		lineWidth += 0.1; printf("lineWidth %f\n", lineWidth);
		break;
	case 'L' :
		lineWidth -= 0.1; printf("lineWidth %f\n", lineWidth);
		break;
	case 't' :
		sharpThreshold += 0.01;
		printf("sharpThreshold %f\n", sharpThreshold*180.0/3.1415928);
		break;
	case 'T' :
		sharpThreshold -= 0.01;
		printf("sharpThreshold %f\n", sharpThreshold*180/3.1515928);
		break;
	case 'm' :
		sharpFlag  = 1 - sharpFlag;
		break;
	case 'o' :
		offset_factor += 0.1;
		printf("offset factor %f\n", offset_factor);
		break;
	case 'O' :
		offset_factor -= 0.1;
		printf("offset factor %f\n", offset_factor);
		break;
	case 'u' :
		offset_units += 0.1;
		printf("offset units %f\n", offset_units);
		break;
	case 'U' :
		offset_units -= 0.1;
		printf("offset units %f\n", offset_units);
		break;
	case ',' :
		zdist += 1;
		break;
	case '.' :
		zdist -= 1;
		break;
	case 'p' :
		perturb();printf("peturbed\n");
		break;
	case '\033': exit(0);
	}
    glutPostRedisplay();
}

// very simple user interface
void mouse(int button, int state, int x, int y)
{
	ox = x; oy = y;	buttonState = button;
}
void mousemotion(int x, int y)
{
	int dx, dy;
	dx = x - ox;
	dy = y - oy;
	
	switch ( buttonState)
	{
	case GLUT_LEFT_BUTTON :
		nAngle += 10*(float) dx/winWidth;
		nAngle2 += 10*(float) dy/winHeight;
		break;
	case GLUT_MIDDLE_BUTTON :
		xdist += 5*dx/40;
		ydist -= 5*dy/40;
		break;
	}
    glutPostRedisplay();
}

void
main(int argc, char **argv)
{
	int val;

	printf("\n\n\nIMP : This program is *only* a demonstration using traditional API\n");
	printf("      Hence, performance of this version is likely to be low\n");
	printf("      But, there is no pre-processing and no connectivity information is used\n");
	printf("      During run time, processing involves only one polygon at a time\n\n");
	printf("      The proposed concept, when supported, is much more efficient\n\n");
	printf("\n\n (If the program complains about lack of Opengl32.dll\n");
	printf("   copy the file, Opengl32.generic.dll to Opengl32.dll in this directory)\n\n\n");

	printf("---Press enter to continue---\n"); getchar();

    glutInit(&argc, argv);
    glutInitWindowSize(winWidth,winHeight);
//    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE|GLUT_STENCIL);
//    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_STENCIL);
    glutCreateWindow("(Click right mouse for menu; keys l,s,z,c,L,S,Z) raskar@cs.unc.edu");
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutReshapeFunc(reshape);
    glutMouseFunc(mouse);
    glutMotionFunc(mousemotion);
	glGetIntegerv(GL_STENCIL_BITS, &val);
	printf("\n --Your graphics support--\n\n OpenGL Version: %s, Renderer: %s,\n Vendor: %s, Stencil bits: %d\n", (char *) glGetString(GL_VERSION),
		glGetString(GL_RENDERER), glGetString(GL_VENDOR), val);
	
	if (val < 6) { printf("\n\n\n YOU DO NOT HAVE SUFFICIENT STENCIL BITS \n");
	printf("\n\n\n Valleys and Intersections will not be rendered correctly \n");}

    glutCreateMenu(menu);
    glutAddMenuEntry("Object (first layer)", OBJ_FRONT);
    glutAddMenuEntry("Object (second layer)", OBJ_BACK);
    glutAddMenuEntry("Object as wireframe", OBJ_WIRED);
    glutAddMenuEntry("Object as wirefilled", OBJ_WIREFILLED);
    //glutAddMenuEntry("1. Wireframe Mode", SIL_LINE);
    //glutAddMenuEntry("2. Z scaled Translation", SIL_ZSCALED);
    //glutAddMenuEntry("3. Fattened Polygons", SIL_FAT);
    glutAddMenuEntry("Silhouettes", SIL_FAT);
    glutAddMenuEntry("Ridges", SHARP);
    glutAddMenuEntry("Valleys", VALLEY);
    glutAddMenuEntry("Intersect", INTERSECT);
    glutAddMenuEntry("Sil and Ridges", SILRIDGE);
    glutAddMenuEntry("Sil, Ridges and Valley", SILRIDGEVALLEY);
    glutAddMenuEntry("Valley and Intersect", VALLEYINTERSECT);
    glutAddMenuEntry("Toggle Object+Sil / Sil", SIL_AND_OBJ);
    glutAddMenuEntry("Toggle Charcoal Style", CHARCOAL);
	glutAddMenuEntry("Model - Face", FACE_MODE);
	glutAddMenuEntry("Model - Venus", VENUS_MODE);
	glutAddMenuEntry("Model - Jeep", CUBE_MODE);
	glutAddMenuEntry("Model - Bunny", BUNNY_MODE);
	glutAddMenuEntry("=Change threshold dihedral angle with keys T or t for ridges and valleys", -1);
	glutAddMenuEntry("=Change width using keys S,s,L,l,U,u", -1);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
	
	#include "disable.h"
	
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
	
	
	// Read in three sample files	
    readModel("face.dat", FACE_MODEL);
	readModel("venus.dat", VENUS_MODEL);
	//readModel("funkyCube.dat", CUBE_MODEL);
	readModel("lrover.dat", CUBE_MODEL);
	//readModel("chair.dat", CUBE_MODEL);
	//readModel("simple2.dat", BUNNY_MODEL);
	readModel("sparks.dat", BUNNY_MODEL);
	
	printf("\n\nLeft mouse button for rotation\n");
	printf("\nMiddle mouse button for translation\n");
	printf("\nRight mouse button for menu\n\n");


    glClearColor(1.0,1.0,1.0,1.0);
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
    gluPerspective(FOV, (float) winWidth/winHeight,1.0,ZFAR);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
    glTranslatef(0,0,-500);
    glutMainLoop();
}


